'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Task2 = require('./Task');

var _Task3 = _interopRequireDefault(_Task2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * PromiseTask
 *
 * Takes a Promise and waits for it to complete.
 * Will attack to finally and complete.
 * Useful for triggers.
 *
 * Similar with TriggerTask, but with set of complete and recover.
 *
 * @param {Promise} promise promise instance to wait for
 * @param {Function} complete function to be called on complete
 * @param {Function} recover function to be called on recover
 */
var PromiseTask = function (_Task) {
  _inherits(PromiseTask, _Task);

  /**
   * PromiseTask constructor
   *
   * @param {Promise} promise promise instance to wait for
   * @param {Function} complete function to be called on complete
   * @param {Function} recover function to be called on recover
   */
  function PromiseTask(promise, complete, recover) {
    _classCallCheck(this, PromiseTask);

    var _this = _possibleConstructorReturn(this, (PromiseTask.__proto__ || Object.getPrototypeOf(PromiseTask)).call(this, undefined, complete, recover));

    _this._promise = undefined;
    _this._setPromise(promise);

    _this._resolved = false;
    return _this;
  }

  /**
   * Runs a task
   *
   * Checks if promise already resolved. If not does nothing.
   *
   * @param {any} args any arguments
   */


  _createClass(PromiseTask, [{
    key: 'run',
    value: function run() {
      this._running = true;

      if (this._resolved) {
        this.complete();
      }
    }

    /**
     * Checks if instance is Promise
     *
     * We do not want promises as a dependency - as it's conditionally used.
     * The checks for promise will be a bit loose - just checking for finally
     * and assuming it's a Promise.
     *
     * @param {Promise} promise promise instance
     * @return {boolean}
     */

  }, {
    key: '_isPromise',
    value: function _isPromise(promise) {
      return promise && (promise.constructor && promise.constructor.name === 'Promise' || promise.finally && promise.finally.constructor.name === 'Function');
    }

    /**
     * Sets promise and attaches to finally
     *
     * @param {Promise} promise promise instance
     */

  }, {
    key: '_setPromise',
    value: function _setPromise(promise) {
      var _this2 = this;

      if (this._isPromise(promise)) {
        this._promise = promise;

        this._promise.finally(function () {
          return _this2._promiseFinally();
        });
      }
    }

    /**
     * Handles promise finally resolved
     */

  }, {
    key: '_promiseFinally',
    value: function _promiseFinally() {
      this._resolved = true;

      if (this._running) {
        this.complete();
      }
    }
  }]);

  return PromiseTask;
}(_Task3.default);

exports.default = PromiseTask;